//-----------------------------------------------
//-----------------------------------------------
#include <SoftwareSerial.h>

#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;

SoftwareSerial bleSerial(A0, A1); // RX, TX
//-----------------------------------------------
//for Speaker
const int buzzer1Port = 5;
const int buzzer2Port = 6;
const int notes[] = {
  131, 147, 165, 175, 196, 220, 247, 262
};
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(bleSerial.available())
  {
    char inChar = bleSerial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//-----------------------------------------------
void setup()
{
  Serial.begin(9600);
  Serial.println("Test Started!");
  bleSerial.begin(9600);
  initFlag();
  initUart();
}

void loop()
{
  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      bleSerial.flush();
      initFlag();
      initUart();
       while(bleSerial.available())
      {
        bleSerial.read();
      }
      checkNextStep();
    break;
    case OPERATION_STEP_2:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        Serial.println("CONNECT OK");
        currentStep++;
      }
      else
      {
        initUart();
        Serial.println("CONNECT ERROR");
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_3:
      if(bleSerial.available())
      {
        char receivedData = bleSerial.read();
        Serial.println("Received 0x" + String(receivedData,HEX));
        Serial.println(receivedData);
        if((receivedData >= '1') && (receivedData <='8'))
        {
          tone(buzzer1Port, notes[receivedData - 0x30 -1], 100);
        }
        //
        if((receivedData >= 'a') && (receivedData <='h'))
        {
          tone(buzzer2Port, notes[receivedData - 0x61], 100);
        }
        if((receivedData >= 'A') && (receivedData <='H'))
        {
          tone(buzzer2Port, notes[receivedData - 0x41], 100);
        }
        //
        delay(500);
      }
    break;

    default:
      if(bleSerial.available())
        Serial.write(bleSerial.read());
      if(Serial.available())
        bleSerial.write(Serial.read());
    break;
  }
}
