//-----------------------------------------------
//-----------------------------------------------
#include <SoftwareSerial.h>

#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;

SoftwareSerial bluetoothSerial(A0, A1); // RX, TX
//-----------------------------------------------
//for FND
int digit[10][4] = {
  { LOW,  LOW,  LOW,  LOW},  //  0, 0, 0, 0 => 0
  { LOW,  LOW,  LOW, HIGH},  //  0, 0, 0, 1 => 1
  { LOW,  LOW, HIGH,  LOW},  //  0, 0, 1, 0 => 2
  { LOW,  LOW, HIGH, HIGH},  //  0, 0, 1, 1 => 3
  { LOW, HIGH,  LOW,  LOW},  //  0, 1, 0, 0 => 4
  { LOW, HIGH,  LOW, HIGH},  //  0, 1, 0, 1 => 5
  { LOW, HIGH, HIGH,  LOW},  //  0, 1, 1, 0 => 6
  { LOW, HIGH, HIGH, HIGH},  //  0, 1, 1, 1 => 7
  {HIGH,  LOW,  LOW,  LOW},  //  1, 0, 0, 0 => 8
  {HIGH,  LOW,  LOW, HIGH}   //  1, 0, 0, 1 => 9
};

int firstFndPins[] = {8, 7, 6, 5};
int secondFndPins[] = {12, 11, 10, 9};
char firstFnd = 0;
char secondFnd = 0;
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(bluetoothSerial.available())
  {
    char inChar = bluetoothSerial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//for FND
void processFnd(char secondValue, char firstValue)
{
  for(int j = 0; j <= 3; j++)
  {
    digitalWrite(secondFndPins[j], digit[secondValue][j]);
    Serial.print(digit[secondValue][j]);
    Serial.print(" ");
  }
  Serial.println(" : secondFnd");

  for(int j = 0; j <= 3; j++)
  {
    digitalWrite(firstFndPins[j], digit[firstValue][j]);
    Serial.print(digit[firstValue][j]);
    Serial.print(" ");
  }
  Serial.println(" : firstFnd");
}
//-----------------------------------------------
void setup()
{
  Serial.begin(9600);
  Serial.println("Test Started!");
  bluetoothSerial.begin(9600);
  initFlag();
  initUart();

  for(int i = 0; i < 4; i++)
  {
    pinMode(secondFndPins[i], OUTPUT);
    pinMode(firstFndPins[i], OUTPUT);
  }
  processFnd(0, 0);
}

void loop()
{
  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      bluetoothSerial.flush();
      initFlag();
      initUart();
      while(bluetoothSerial.available())
      {
        bluetoothSerial.read();
      }
      checkNextStep();
    break;
    case OPERATION_STEP_2:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        Serial.println("CONNECT OK");
        currentStep++;
      }
      else
      {
        initUart();
        Serial.println("CONNECT ERROR");
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_3:
      if(bluetoothSerial.available())
      {
        firstFnd = bluetoothSerial.read();
        // 0 부터 9 사이의 ascii 문자인가?
        if(isDigit(firstFnd))
        {
          Serial.print("---------received: ");
          Serial.println(firstFnd);
          firstFnd = firstFnd - '0';
          processFnd(secondFnd, firstFnd);
          secondFnd = firstFnd;
        }
      }
    break;

    default:
      if(bluetoothSerial.available())
        Serial.write(bluetoothSerial.read());
      if(Serial.available())
        bluetoothSerial.write(Serial.read());
    break;
  }
}
