//-----------------------------------------------
//-----------------------------------------------
#include <SoftwareSerial.h>

#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;

SoftwareSerial bluetoothSerial(A0, A1); // RX, TX
//-----------------------------------------------
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(bluetoothSerial.available())
  {
    char inChar = bluetoothSerial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//-----------------------------------------------
void setup()
{
  Serial.begin(9600);
  Serial.println("Test Started!");
  bluetoothSerial.begin(9600);
  initFlag();
  initUart();

  for(int i = 5; i < 9; i++)
  {
    pinMode(i, INPUT);
  }

  for(int i = 9; i < 13; i++)
  {
    pinMode(i, OUTPUT);
    digitalWrite(i,HIGH);
  }
}

void loop()
{
  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      bluetoothSerial.flush();
      initFlag();
      initUart();
      while(bluetoothSerial.available())
      {
        bluetoothSerial.read();
      }
      checkNextStep();
    break;
    case OPERATION_STEP_2:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        Serial.println("CONNECT OK");
        currentStep++;
      }
      else
      {
        initUart();
        Serial.println("CONNECT ERROR");
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_3:
    {
      for(int i = 0; i<4; i++)
      {
        if(!digitalRead(i+5))
        {
          char sendingData = i+0x30+1;
          bluetoothSerial.write(sendingData);
          Serial.println("Sending 0x" + String(sendingData,HEX));
          delay(100);
        }
      }
      
      if(bluetoothSerial.available())
      {
        char receivedData = bluetoothSerial.read();
        Serial.println("Received 0x" + String(receivedData,HEX));
        if((receivedData >= '5') && (receivedData <='8'))
        {
          receivedData = receivedData - 0x30 + 4;
          if(digitalRead(receivedData))
            digitalWrite(receivedData,LOW);
          else
            digitalWrite(receivedData,HIGH);
        }
      }
    }
    break;

    default:
      if(bluetoothSerial.available())
        Serial.write(bluetoothSerial.read());
      if(Serial.available())
        bluetoothSerial.write(Serial.read());
    break;
  }
}
