//-----------------------------------------------
//-----------------------------------------------
#include <SoftwareSerial.h>

#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;

SoftwareSerial bluetoothSerial(A0, A1); // RX, TX
//-----------------------------------------------
//for Motor
const int motor1ForwardPwm = 9;
const int motor1ForwardSignal = 12;
const int motor1BackwardPwm = 10;
const int motor1BackwardSignal = 11;

const int motor2ForwardPwm = 5;
const int motor2ForwardSignal = 8;
const int motor2BackwardPwm = 6;
const int motor2BackwardSignal = 7;

const int motor1Forward = '1';
const int motor1Backward = '2';
const int motor1Up = '3';
const int motor1Down = '4';

const int motor2Forward = '5';
const int motor2Backward = '6';
const int motor2Up = '7';
const int motor2Down = '8';

unsigned int motor1Gear = motor1Forward;
unsigned int motor1Speed = 0;
unsigned int motor1SpeedValue = 0;

unsigned int motor2Gear = motor2Forward;
unsigned int motor2Speed = 0;
unsigned int motor2SpeedValue = 0;
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(bluetoothSerial.available())
  {
    char inChar = bluetoothSerial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//for Motor
void controlMotor1()
{
  if(motor1Gear == motor1Forward)
  {
    if(motor1Speed == motor1Up)
    {
      if(motor1SpeedValue < 226)
        motor1SpeedValue = motor1SpeedValue + 25;
    }
    else if(motor1Speed == motor1Down)
    {
      if(motor1SpeedValue > 24)
        motor1SpeedValue = motor1SpeedValue - 25;
    }
    //
    if(motor1SpeedValue == 0)
      digitalWrite(motor1ForwardSignal, HIGH);
    else
      digitalWrite(motor1ForwardSignal, LOW);
    analogWrite(motor1ForwardPwm, motor1SpeedValue);
    //
    digitalWrite(motor1BackwardSignal, HIGH);
    analogWrite(motor1BackwardPwm, 0);
  }
  else if(motor1Gear == motor1Backward)
  {
    if(motor1Speed == motor1Up)
    {
      if(motor1SpeedValue < 226)
        motor1SpeedValue = motor1SpeedValue + 25;
    }
    else if(motor1Speed == motor1Down)
    {
      if(motor1SpeedValue > 24)
        motor1SpeedValue = motor1SpeedValue - 25;
    }
    //
    digitalWrite(motor1ForwardSignal, HIGH);
    analogWrite(motor1ForwardPwm, 0);
    //
    if(motor1SpeedValue == 0)
      digitalWrite(motor1BackwardSignal, HIGH);
    else
      digitalWrite(motor1BackwardSignal, LOW);
    analogWrite(motor1BackwardPwm, motor1SpeedValue);
  }
}
//
void controlMotor2()
{
  if(motor2Gear == motor2Forward)
  {
    if(motor2Speed == motor2Up)
    {
      if(motor2SpeedValue < 226)
        motor2SpeedValue = motor2SpeedValue + 25;
    }
    else if(motor2Speed == motor2Down)
    {
      if(motor2SpeedValue > 24)
        motor2SpeedValue = motor2SpeedValue - 25;
    }
    //
    if(motor2SpeedValue == 0)
      digitalWrite(motor2ForwardSignal, HIGH);
    else
      digitalWrite(motor2ForwardSignal, LOW);
    analogWrite(motor2ForwardPwm, motor2SpeedValue);
    //
    digitalWrite(motor2BackwardSignal, HIGH);
    analogWrite(motor2BackwardPwm, 0);
  }
  else if(motor2Gear == motor2Backward)
  {
    if(motor2Speed == motor2Up)
    {
      if(motor2SpeedValue < 226)
        motor2SpeedValue = motor2SpeedValue + 25;
    }
    else if(motor2Speed == motor2Down)
    {
      if(motor2SpeedValue > 24)
        motor2SpeedValue = motor2SpeedValue - 25;
    }
    //
    digitalWrite(motor2ForwardSignal, HIGH);
    analogWrite(motor2ForwardPwm, 0);
    //
    if(motor2SpeedValue == 0)
      digitalWrite(motor2BackwardSignal, HIGH);
    else
      digitalWrite(motor2BackwardSignal, LOW);
    analogWrite(motor2BackwardPwm, motor2SpeedValue);
  }
}
//-----------------------------------------------
void setup()
{
  Serial.begin(9600);
  Serial.println("Test Started!");
  bluetoothSerial.begin(9600);
  initFlag();
  initUart();

  for(int i = 5; i <= 12; i++)
  {
    pinMode(i, OUTPUT);
  }

  digitalWrite(motor1ForwardPwm, LOW);
  digitalWrite(motor1BackwardPwm, LOW);
  digitalWrite(motor1BackwardSignal, HIGH);
  digitalWrite(motor1ForwardSignal, HIGH);
  
  digitalWrite(motor2ForwardPwm, LOW);
  digitalWrite(motor2BackwardPwm, LOW);
  digitalWrite(motor2BackwardSignal, HIGH);
  digitalWrite(motor2ForwardSignal, HIGH);
}

void loop()
{
  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      bluetoothSerial.flush();
      initFlag();
      initUart();
      while(bluetoothSerial.available())
      {
        bluetoothSerial.read();
      }
      checkNextStep();
    break;
    case OPERATION_STEP_2:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        Serial.println("CONNECT OK");
        currentStep++;
      }
      else
      {
        initUart();
        Serial.println("CONNECT ERROR");
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_3:
      if(bluetoothSerial.available())
      {
        char receivedData = bluetoothSerial.read();
        Serial.print("Received: ");
        Serial.println(receivedData);
        //
        //Received Motor 1 Control Data
        if((receivedData == motor1Forward)
          ||(receivedData == motor1Backward))
        {
          motor1Gear = receivedData;
          motor1Speed = 0;
          controlMotor1();
        }
    
        if((receivedData == motor1Up)
          ||(receivedData == motor1Down))
        {
          motor1Speed = receivedData;
          controlMotor1();
        }
        //
        //Received Motor 2 Control Data
        if((receivedData == motor2Forward)
          ||(receivedData == motor2Backward))
        {
          motor2Gear = receivedData;
          motor2Speed = 0;
          controlMotor2();
        }
    
        if((receivedData == motor2Up)
          ||(receivedData == motor2Down))
        {
          motor2Speed = receivedData;
          controlMotor2();
        }
      }
    break;

    default:
      if(bluetoothSerial.available())
        Serial.write(bluetoothSerial.read());
      if(Serial.available())
        bluetoothSerial.write(Serial.read());
    break;
  }
}

