//-----------------------------------------------
//-----------------------------------------------
#include <SoftwareSerial.h>

#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;

SoftwareSerial bluetoothSerial(A0, A1); // RX, TX
//-----------------------------------------------
//for PWM Speaker or Motor
const int pwm1Port = 5;
const int pwm2Port = 6;

const int pwm1Up = '1';
const int pwm1Down = '2';

const int pwm2Up = '3';
const int pwm2Down = '4';

unsigned int pwm1Speed = 0;
unsigned int pwm1SpeedValue = 0;

unsigned int pwm2Speed = 0;
unsigned int pwm2SpeedValue = 0;
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(bluetoothSerial.available())
  {
    char inChar = bluetoothSerial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//for PWM
void controlPWM1()
{
  if(pwm1Speed == pwm1Up)
  {
    if(pwm1SpeedValue < 226)
      pwm1SpeedValue = pwm1SpeedValue + 25;
  }
  else if(pwm1Speed == pwm1Down)
  {
    if(pwm1SpeedValue > 24)
      pwm1SpeedValue = pwm1SpeedValue - 25;
  }
  //
  analogWrite(pwm1Port, pwm1SpeedValue);
}
//
void controlPWM2()
{
  if(pwm2Speed == pwm2Up)
  {
    if(pwm2SpeedValue < 226)
      pwm2SpeedValue = pwm2SpeedValue + 25;
  }
  else if(pwm2Speed == pwm2Down)
  {
    if(pwm2SpeedValue > 24)
      pwm2SpeedValue = pwm2SpeedValue - 25;
  }
  //
  analogWrite(pwm2Port, pwm2SpeedValue);
}
//-----------------------------------------------
void setup()
{
  Serial.begin(9600);
  Serial.println("Test Started!");
  bluetoothSerial.begin(9600);
  initFlag();
  initUart();

  for(int i = 5; i <= 6; i++)
  {
    pinMode(i, OUTPUT);
    digitalWrite(i, LOW);
  }
}

void loop()
{
  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      bluetoothSerial.flush();
      initFlag();
      initUart();
       while(bluetoothSerial.available())
      {
        bluetoothSerial.read();
      }
      checkNextStep();
    break;
    case OPERATION_STEP_2:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        Serial.println("CONNECT OK");
        currentStep++;
      }
      else
      {
        initUart();
        Serial.println("CONNECT ERROR");
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_3:
      if(bluetoothSerial.available())
      {
        char receivedData = bluetoothSerial.read();
        Serial.println("Received 0x" + String(receivedData,HEX));
        if((receivedData >= '1') && (receivedData <='2'))
        {
          pwm1Speed = receivedData;
          controlPWM1();
        }
        //
        if((receivedData >= '3') && (receivedData <='4'))
        {
          pwm2Speed = receivedData;
          controlPWM2();
        }
      }
    break;

    default:
      if(bluetoothSerial.available())
        Serial.write(bluetoothSerial.read());
      if(Serial.available())
        bluetoothSerial.write(Serial.read());
    break;
  }
}
