//-----------------------------------------------
//for LCD
#include <LiquidCrystal.h>
//-----------------------------------------------
#include <SoftwareSerial.h>

#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;

SoftwareSerial bluetoothSerial(A0, A1); // RX, TX
//-----------------------------------------------
//for LCD
unsigned int displayCount = 0;

LiquidCrystal lcd(10, 9, 8, 7, 6, 5);
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(bluetoothSerial.available())
  {
    char inChar = bluetoothSerial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//-----------------------------------------------
void setup()
{
  Serial.begin(9600);
  Serial.println("Test Started!");
  bluetoothSerial.begin(9600);
  initFlag();
  initUart();

  lcd.begin(16, 2);
  lcd.print("Hello World!");
  lcd.setCursor(0,0);
  lcd.blink();
}

void loop()
{
  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      bluetoothSerial.flush();
      initFlag();
      initUart();
      while(bluetoothSerial.available())
      {
        bluetoothSerial.read();
      }
      checkNextStep();
    break;
    case OPERATION_STEP_2:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        Serial.println("CONNECT OK");
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.blink();
        currentStep++;
      }
      else
      {
        initUart();
        Serial.println("CONNECT ERROR");
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_3:
      if(bluetoothSerial.available())
      {
        char receivedData = bluetoothSerial.read();
        if(receivedData == 0x0d)
        {
          lcd.clear();
          lcd.setCursor(0,0);
          displayCount = 0;
        }
        else
        {
          lcd.write(receivedData);
          displayCount++;
        }

        if(displayCount == 16)
        {
          lcd.setCursor(0,1);
          lcd.blink();
        }
        else if(displayCount > 32)
        {
          displayCount = 1;
          lcd.clear();
          lcd.setCursor(0,0);
          lcd.blink();
          lcd.write(receivedData);
        }
      }
    break;

    default:
      if(bluetoothSerial.available())
        Serial.write(bluetoothSerial.read());
      if(Serial.available())
        bluetoothSerial.write(Serial.read());
    break;
  }
}
