//-----------------------------------------------
//for PS2 Keyboard
#include <PS2Keyboard.h>
//-----------------------------------------------
#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3
#define OPERATION_STEP_4  4
#define OPERATION_STEP_5  5
#define OPERATION_STEP_6  6
#define OPERATION_STEP_7  7
#define OPERATION_STEP_8  8
#define OPERATION_STEP_9  9
#define OPERATION_STEP_10  10
#define OPERATION_STEP_11  11
#define OPERATION_STEP_12  12
#define OPERATION_STEP_13  13

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;
//-----------------------------------------------
//for PS2 Keyboard

const int DataPin = 3;
const int IRQpin =  2;

PS2Keyboard keyboard;

unsigned char serialHidTable[][2] = {
  {0x01,0x90},  /*  SOH => F1: used change korean/english of Galaxy & PC(or Note book)  */
  {0x02,0x2C},  /*  STX => F2: used change korean/english of iPad         */
  {0x12,0x00},  /*  Alt => F3: used change korean/english of Note book          */
  //
  {0x08,0x2A},  /*  back space      */
  {0x09,0x2B},  /*  tab         */
  {0x0D,0x28},  /*  carriage return   */
  {0x1B,0x29},  /*  ESC         */
  {0x20,0x2C},  /*  space       */
  {0x27,0x34},  /*  '         */
  {0x2C,0x36},  /*  ,         */
  {0x2D,0x2D},  /*  -         */
  {0x2E,0x37},  /*  .         */
  {0x2F,0x38},  /*  /         */
  {0x30,0x27},  /*  0         */
  {0x31,0x1E},  /*  1         */
  {0x32,0x1F},  /*  2         */
  {0x33,0x20},  /*  3         */
  {0x34,0x21},  /*  4         */
  {0x35,0x22},  /*  5         */
  {0x36,0x23},  /*  6         */
  {0x37,0x24},  /*  7         */
  {0x38,0x25},  /*  8         */
  {0x39,0x26},  /*  9         */
  {0x3B,0x33},  /*  ;         */
  {0x3D,0x2E},  /*  =         */
  {0x5B,0x2F},  /*  [         */
  {0x5C,0x31},  /*  \         */
  {0x5D,0x30},  /*  ]         */
  {0x60,0x35},  /*  `         */
  {0x61,0x04},  /*  a         */
  {0x62,0x05},  /*  b         */
  {0x63,0x06},  /*  c         */
  {0x64,0x07},  /*  d         */
  {0x65,0x08},  /*  e         */
  {0x66,0x09},  /*  f         */
  {0x67,0x0A},  /*  g         */
  {0x68,0x0B},  /*  h         */
  {0x69,0x0C},  /*  i         */
  {0x6A,0x0D},  /*  j         */
  {0x6B,0x0E},  /*  k         */
  {0x6C,0x0F},  /*  l         */
  {0x6D,0x10},  /*  m         */
  {0x6E,0x11},  /*  n         */
  {0x6F,0x12},  /*  o         */
  {0x70,0x13},  /*  p         */
  {0x71,0x14},  /*  q         */
  {0x72,0x15},  /*  r         */
  {0x73,0x16},  /*  s         */
  {0x74,0x17},  /*  t         */
  {0x75,0x18},  /*  u         */
  {0x76,0x19},  /*  v         */
  {0x77,0x1A},  /*  w         */
  {0x78,0x1B},  /*  x         */
  {0x79,0x1C},  /*  y         */
  {0x7A,0x1D},  /*  z         */
  //
  {0x21,0x1E},  /*  ! => Shift + 1      */
  {0x22,0x34},  /*  " => Shift + '      */
  {0x23,0x20},  /*  # => Shift + 3    */
  {0x24,0x21},  /*  $ => Shift + 4      */
  {0x25,0x22},  /*  % => Shift + 5    */
  {0x26,0x24},  /*  & => Shift + 7    */
  //
  {0x28,0x26},  /*  ( => Shift + 9      */
  {0x29,0x27},  /*  ) => Shift + 0      */
  {0x2A,0x25},  /*  * => Shift + 8      */
  {0x2B,0x2E},  /*  + => Shift + =    */
  //
  {0x3A,0x33},  /*  : => Shift + ;      */
  //
  {0x3C,0x36},  /*  < => Shift + ,      */
  //
  {0x3E,0x37},  /*  > => Shift + .      */
  {0x3F,0x38},  /*  ? => Shift + /      */
  {0x40,0x1F},  /*  @ => Shift + 2    */
  {0x41,0x04},  /*  A => Shift + a    */
  {0x42,0x05},  /*  B => Shift + b    */
  {0x43,0x06},  /*  C => Shift + c    */
  {0x44,0x07},  /*  D => Shift + d    */
  {0x45,0x08},  /*  E => Shift + e      */
  {0x46,0x09},  /*  F => Shift + f      */
  {0x47,0x0A},  /*  G => Shift + g    */
  {0x48,0x0B},  /*  H => Shift + h    */
  {0x49,0x0C},  /*  I => Shift + i      */
  {0x4A,0x0D},  /*  J => Shift + j      */
  {0x4B,0x0E},  /*  K => Shift + k    */
  {0x4C,0x0F},  /*  L => Shift + l      */
  {0x4D,0x10},  /*  M => Shift + m    */
  {0x4E,0x11},  /*  N => Shift + n    */
  {0x4F,0x12},  /*  O => Shift + o    */
  {0x50,0x13},  /*  P => Shift + p      */
  {0x51,0x14},  /*  Q => Shift + q    */
  {0x52,0x15},  /*  R => Shift + r      */
  {0x53,0x16},  /*  S => Shift + s      */
  {0x54,0x17},  /*  T => Shift + t      */
  {0x55,0x18},  /*  U => Shift + u    */
  {0x56,0x19},  /*  V => Shift + v    */
  {0x57,0x1A},  /*  W => Shift + w    */
  {0x58,0x1B},  /*  X => Shift + x    */
  {0x59,0x1C},  /*  Y => Shift + y      */
  {0x5A,0x1D},  /*  Z => Shift + z      */
  //
  {0x5E,0x23},  /*  ^ => Shift + 6    */
  {0x5F,0x2D},  /*  _ => Shift + -      */
  //
  {0x7B,0x2F},  /*  { => Shift + [      */
  {0x7C,0x31},  /*  | => Shift + \      */
  {0x7D,0x30},  /*  } => Shift + ]      */
  {0x7E,0x35},  /*  ~ => Shift + `    */
  //
  {0x19, 0x4B},  /*  Page Up    */
  {0x1A, 0x4E},  /*  Page Down    */
  {0x0B, 0x52},  /*  Up Arrow    */
  {0x07, 0x50},  /*  Left Arrow    */
  {0x0A, 0x51},  /*  Down Arrow    */
  {0x15, 0x4F},  /*  Right Arrow    */
  {0x7F, 0x4C},  /*  Delete    */
  //
  {0,0}
};
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(Serial.available())
  {
    char inChar = Serial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//for PS2 Keyboard
void sendHidReport(unsigned char keyValue,unsigned char modifierDataValue)
{
  //for Chage Korean/English of iPad
  if(modifierDataValue == 0x08)
  {
    Serial.write((byte)0x0c);
    Serial.write((byte)0x00);
    Serial.write((byte)0xa1);
    Serial.write((byte)0x01);
    //
    Serial.write((byte)modifierDataValue);
    //
    Serial.write((byte)0x00);
    //
    Serial.write((byte)0x00);//key value
    //
    Serial.write((byte)0x00);
    Serial.write((byte)0x00);
    Serial.write((byte)0x00);
    Serial.write((byte)0x00);
    Serial.write((byte)0x00);
    //
    delay(10);
  }
  //
  Serial.write((byte)0x0c);
  Serial.write((byte)0x00);
  Serial.write((byte)0xa1);
  Serial.write((byte)0x01);
  //
  Serial.write((byte)modifierDataValue);
  //
  Serial.write((byte)0x00);
  //
  Serial.write((byte)keyValue);
  //
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
  //
  //keyValue cancellation
  delay(10);
  //
  Serial.write((byte)0x0c);
  Serial.write((byte)0x00);
  Serial.write((byte)0xa1);
  Serial.write((byte)0x01);
  //
  Serial.write((byte)modifierDataValue);
  //
  Serial.write((byte)0x00);
  //
  Serial.write((byte)0x00);//keyValue
  //
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
  Serial.write((byte)0x00);
}
//-----------------------------------------------
void setup() {
  keyboard.begin(DataPin, IRQpin);
  Serial.begin(115200);
  initFlag();
  initUart();
}

void loop() {
  unsigned int loopCount;
  unsigned char ps2Data;
  unsigned char modifierData;

  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      Serial.flush();
      initFlag();
      initUart();
      while(Serial.available())
      {
        Serial.read();
      }
      currentStep++;
    break;
    case OPERATION_STEP_2:
      delay(300);
      initUart();
      Serial.print("at\r");
      checkNextStep();
    break;
    case OPERATION_STEP_3:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("at+btinfo?6\r");
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_4:
      if(uartString.startsWith("\r\nSLAVE,"))
      {
        delay(300);
        initUart();
        Serial.print("at+btinfo?1\r");
        currentStep = OPERATION_STEP_5;
        checkNextStep();
      }
      else
      {
        delay(300);
        initUart();
        Serial.print("at+btrole=s\r");
        checkNextStep();
      }
    break;
    case OPERATION_STEP_5:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("atz\r");
        currentStep = OPERATION_STEP_0;
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_6:
      if(uartString.startsWith("\r\n0000,"))
      {
        delay(300);
        initUart();
        Serial.print("atz\r");
        currentStep = OPERATION_STEP_7;
        checkNextStep();
      }
      else
      {
        delay(300);
        initUart();
        Serial.print("at+btkey=0000\r");
        checkNextStep();
      }
    break;
    case OPERATION_STEP_7:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("atz\r");
        currentStep = OPERATION_STEP_0;
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    //HID는 ATZ 명령을 받으면 OK를 출력한다.
    case OPERATION_STEP_8:
      if(uartString.equals("\r\nOK\r\n"))
      {
        initUart();
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_9:
      if(uartString.equals("\r\nBTWIN HID Slave mode start\r\n"))
      {
        initUart();
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_10:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("at+btscan\r");
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_11:
      if(uartString.equals("\r\nOK\r\n"))
      {
        initUart();
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_12:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        currentStep++;
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_13:
      if (keyboard.available())
      {
        // read the next key
        ps2Data = keyboard.read();
        
        for(loopCount = 0; serialHidTable[loopCount][0] != ps2Data && serialHidTable[loopCount][0]; loopCount++);
        
        if((serialHidTable[loopCount][0] == ps2Data)&&(ps2Data !=0))
        {
          //Shift가 눌린 상태의 keyValue
          if(((ps2Data  > 0x20) && (ps2Data < 0x27))
            ||((ps2Data  > 0x27) && (ps2Data < 0x2C))
            ||(ps2Data == 0x3A)
            ||(ps2Data == 0x3C)
            ||((ps2Data  > 0x3D) && (ps2Data < 0x5B))
            ||((ps2Data  > 0x5D) && (ps2Data < 0x60))
            ||((ps2Data  > 0x7A) && (ps2Data < 0x7F)))
          {
            modifierData = 0x02;//Left Shift ON
          }
          //F3키가 눌린 상태의 keyValue: Alt Key 대치. Note Book의 한/영 전환에 사용.
          else if(ps2Data == 0x12)
          {
            modifierData = 0x40;//Right Alt ON
          }
          //F2키가 눌린 상태의 keyValue: iPad의 한/영 전환에 사용.
          else if(ps2Data == 0x02)
          {
            modifierData = 0x08;//Left GUI ON
          }
          else
            modifierData = 0x00;
          //
          sendHidReport(serialHidTable[loopCount][1], modifierData);
        }
      }
    break;

    default:
    break;
  }
}
