//-----------------------------------------------
//for PS2 Mouse
#define MDATA 6
#define MCLK 5
//-----------------------------------------------
#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3
#define OPERATION_STEP_4  4
#define OPERATION_STEP_5  5
#define OPERATION_STEP_6  6
#define OPERATION_STEP_7  7
#define OPERATION_STEP_8  8
#define OPERATION_STEP_9  9
#define OPERATION_STEP_10  10
#define OPERATION_STEP_11  11
#define OPERATION_STEP_12  12
#define OPERATION_STEP_13  13

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;
//-----------------------------------------------
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(Serial.available())
  {
    char inChar = Serial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//for PS2 Mouse
void gohi(int pin)
{
  pinMode(pin, INPUT);
  digitalWrite(pin, HIGH);
}

void golo(int pin)
{
  pinMode(pin, OUTPUT);
  digitalWrite(pin, LOW);
}

void mouse_write(char data)
{
  char i;
  char parity = 1;

  /* put pins in output mode */
  gohi(MDATA);
  gohi(MCLK);
  delayMicroseconds(300);
  golo(MCLK);
  delayMicroseconds(300);
  golo(MDATA);
  delayMicroseconds(10);
  /* start bit */
  gohi(MCLK);
  /* wait for mouse to take control of clock); */
  while (digitalRead(MCLK) == HIGH)
    ;
  /* clock is low, and we are clear to send data */
  for (i=0; i < 8; i++) {
    if (data & 0x01) {
      gohi(MDATA);
    }
    else {
      golo(MDATA);
    }
    /* wait for clock cycle */
    while (digitalRead(MCLK) == LOW)
      ;
    while (digitalRead(MCLK) == HIGH)
      ;
    parity = parity ^ (data & 0x01);
    data = data >> 1;
  }
  /* parity */
  if (parity) {
    gohi(MDATA);
  }
  else {
    golo(MDATA);
  }
  while (digitalRead(MCLK) == LOW)
    ;
  while (digitalRead(MCLK) == HIGH)
    ;
  /* stop bit */
  gohi(MDATA);
  delayMicroseconds(50);
  while (digitalRead(MCLK) == HIGH)
    ;
  /* wait for mouse to switch modes */
  while ((digitalRead(MCLK) == LOW) || (digitalRead(MDATA) == LOW))
    ;
  /* put a hold on the incoming data. */
  golo(MCLK);
}

char mouse_read(void)
{
  char data = 0x00;
  int i;
  char bit = 0x01;

  /* start the clock */
  gohi(MCLK);
  gohi(MDATA);
  delayMicroseconds(50);
  while (digitalRead(MCLK) == HIGH)
    ;
  delayMicroseconds(5);  /* not sure why */
  while (digitalRead(MCLK) == LOW) /* eat start bit */
    ;
  for (i=0; i < 8; i++) {
    while (digitalRead(MCLK) == HIGH)
      ;
    if (digitalRead(MDATA) == HIGH) {
      data = data | bit;
    }
    while (digitalRead(MCLK) == LOW)
      ;
    bit = bit << 1;
  }
  /* eat parity bit, which we ignore */
  while (digitalRead(MCLK) == HIGH)
    ;
  while (digitalRead(MCLK) == LOW)
    ;
  /* eat stop bit */
  while (digitalRead(MCLK) == HIGH)
    ;
  while (digitalRead(MCLK) == LOW)
    ;
  /* put a hold on the incoming data. */
  golo(MCLK);
  return data;
}

void mouse_init()
{
  char mouseId;
   
  gohi(MCLK);
  gohi(MDATA);
  //Sending reset to mouse
  mouse_write(0xff);
  mouse_read();  /* ack byte */
  //Read ack byte1
  mouse_read();  /* blank */
  mouse_read();  /* blank */
  //Setting sample rate 200
  mouse_write(0xf3);  /* Set rate command */
  mouse_read();  /* ack */
  mouse_write(0xC8);  /* Set rate command */
  mouse_read();  /* ack */
  //Setting sample rate 100
  mouse_write(0xf3);  /* Set rate command */
  mouse_read();  /* ack */
  mouse_write(0x64);  /* Set rate command */
  mouse_read();  /* ack */
  //Setting sample rate 80
  mouse_write(0xf3);  /* Set rate command */
  mouse_read();  /* ack */
  mouse_write(0x50);  /* Set rate command */
  mouse_read();  /* ack */
  //Read device type
  mouse_write(0xf2);  /* Set rate command */
  mouse_read();  /* ack */
  mouse_read();  /* mouse id, if this value is 0x00 mouse is standard, if it is 0x03 mouse is Intellimouse */
  //Setting wheel
  mouse_write(0xe8);  /* Set wheel resolution */
  mouse_read();  /* ack */
  mouse_write(0x03);  /* 8 counts per mm */
  mouse_read();  /* ack */
  mouse_write(0xe6);  /* scaling 1:1 */
  mouse_read();  /* ack */
  mouse_write(0xf3);  /* Set sample rate */
  mouse_read();  /* ack */
  mouse_write(0x28);  /* Set sample rate */
  mouse_read();  /* ack */
  mouse_write(0xf4);  /* Enable device */
  mouse_read();  /* ack */

  //Sending remote mode code
  mouse_write(0xf0);  /* remote mode */
  mouse_read();  /* ack */
  //Read ack byte2
  delayMicroseconds(100);
}
//-----------------------------------------------
void setup()
{
  Serial.begin(115200);
  initFlag();
  initUart();
}

void loop()
{
  char mstat;
  char mx;
  char my;
  char mz;

  static char old_mstat;
  static char old_mx;
  static char old_my;
  static char old_mz;

  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      delay(2000);
      Serial.flush();
      initFlag();
      initUart();
      while(Serial.available())
      {
        Serial.read();
      }
      currentStep++;
    break;
    case OPERATION_STEP_2:
      delay(300);
      initUart();
      Serial.print("at\r");
      checkNextStep();
    break;
    case OPERATION_STEP_3:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("at+btinfo?6\r");
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_4:
      if(uartString.startsWith("\r\nSLAVE,"))
      {
        delay(300);
        initUart();
        Serial.print("at+btinfo?1\r");
        currentStep = OPERATION_STEP_5;
        checkNextStep();
      }
      else
      {
        delay(300);
        initUart();
        Serial.print("at+btrole=s\r");
        checkNextStep();
      }
    break;
    case OPERATION_STEP_5:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("atz\r");
        currentStep = OPERATION_STEP_0;
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_6:
      if(uartString.startsWith("\r\n0000,"))
      {
        delay(300);
        initUart();
        Serial.print("atz\r");
        currentStep = OPERATION_STEP_7;
        checkNextStep();
      }
      else
      {
        delay(300);
        initUart();
        Serial.print("at+btkey=0000\r");
        checkNextStep();
      }
    break;
    case OPERATION_STEP_7:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("atz\r");
        currentStep = OPERATION_STEP_0;
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    //HID는 ATZ 명령을 받으면 OK를 출력한다.
    case OPERATION_STEP_8:
      if(uartString.equals("\r\nOK\r\n"))
      {
        initUart();
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_9:
      if(uartString.equals("\r\nBTWIN HID Slave mode start\r\n"))
      {
        initUart();
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_10:
      if(uartString.equals("\r\nOK\r\n"))
      {
        delay(300);
        initUart();
        Serial.print("at+btscan\r");
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_11:
      if(uartString.equals("\r\nOK\r\n"))
      {
        initUart();
        checkNextStep();
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_12:
      if(uartString.startsWith("\r\nCONNECT "))
      {
        delay(300);
        initUart();
        mouse_init();
        currentStep++;
      }
      else
      {
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_13:
      /* get a reading from the mouse */
      mouse_write(0xeb);  /* give me data! */
      mouse_read();      /* ignore ack */
      mstat = mouse_read();
      mx = mouse_read();
      my = mouse_read();
      mz = mouse_read();
  
      if((mstat!=old_mstat)||(mx!= old_mx)||(my!= old_my)||(mz!= old_mz))
      {
        Serial.write((byte)0x08);
        Serial.write((byte)0x00);
        Serial.write((byte)0xa1);
        Serial.write((byte)0x02);
        //mouse_button
        Serial.write((byte)mstat);
        //mouse_x
        Serial.write((byte)mx);
        //mouse_y
        Serial.write((byte)~(my)+1);
        //mouse_wheel
        Serial.write((byte)~(mz)+1);
        //
        old_mstat = mstat;
        old_mx = mx;
        old_my = my;
        old_mz = mz;
      }
    break;

    default:
    break;
  }
}
