//-----------------------------------------------
//-----------------------------------------------
#include <SoftwareSerial.h>

#define OPERATION_STEP_0  0
#define OPERATION_STEP_1  1
#define OPERATION_STEP_2  2
#define OPERATION_STEP_3  3
#define OPERATION_STEP_4  4
#define OPERATION_STEP_5  5
#define OPERATION_STEP_6  6
#define OPERATION_STEP_7  7
#define OPERATION_STEP_8  8
#define OPERATION_STEP_9  9
#define OPERATION_STEP_10  10
#define OPERATION_STEP_11  11

#define OPERATION_STEP_ERROR  100

String uartString = "";
unsigned int currentStep;
unsigned int oldStep;

SoftwareSerial bluetoothSerial(A0, A1); // RX, TX
//-----------------------------------------------
//-----------------------------------------------
void initUart()
{
  uartString = "";
}

void initFlag()
{
  currentStep = OPERATION_STEP_1;
  oldStep = OPERATION_STEP_1;
}

void checkNextStep()
{
  oldStep = currentStep;
  currentStep = OPERATION_STEP_0;
}

void returnOldStep()
{
  currentStep = oldStep;
  currentStep++;
}

void checkCrLfProcess()
{
  while(bluetoothSerial.available())
  {
    char inChar = bluetoothSerial.read();
    uartString += inChar;
    //
    if( uartString.length() > 4
     && uartString.startsWith("\r\n")
     && uartString.endsWith("\r\n") )
    {
      returnOldStep();
      break;
    }
  }
}
//-----------------------------------------------
//-----------------------------------------------
void setup()
{
  Serial.begin(9600);
  Serial.println("A2DP Shield Test Started!");
  bluetoothSerial.begin(9600);
  initFlag();
  initUart();
}

void loop()
{
  switch(currentStep)
  {
    case OPERATION_STEP_0:
      checkCrLfProcess();
    break;
    case OPERATION_STEP_1:
      Serial.println("[TP 01]");
      delay(2000);
      bluetoothSerial.flush();
      initFlag();
      initUart();
      while(bluetoothSerial.available())
      {
        bluetoothSerial.read();
      }
      currentStep++;
    break;
    case OPERATION_STEP_2:
      Serial.println("[TP 02]");
      delay(300);
      initUart();
      bluetoothSerial.print("at\r");
      checkNextStep();
    break;
    case OPERATION_STEP_3:
      if(uartString.equals("\r\nOK\r\n"))
      {
        Serial.println("[TP 03]");
        delay(300);
        initUart();
        bluetoothSerial.print("at+btinfo?6\r");
        checkNextStep();
      }
      else
      {
        Serial.println("[ERROR_03]");
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_4:
      if(uartString.startsWith("\r\nSLAVE,"))
      {
        Serial.println("[TP 04-1]");
        delay(300);
        initUart();
        bluetoothSerial.print("at+btinfo?1\r");
        currentStep = OPERATION_STEP_5;
        checkNextStep();
      }
      else
      {
        Serial.println("[TP 04-2]");
        delay(300);
        initUart();
        bluetoothSerial.print("at+btrole=s\r");
        checkNextStep();
      }
    break;
    case OPERATION_STEP_5:
      if(uartString.equals("\r\nOK\r\n"))
      {
        Serial.println("[TP 05]");
        delay(300);
        initUart();
        bluetoothSerial.print("atz\r");
        currentStep = OPERATION_STEP_0;
        checkNextStep();
      }
      else
      {
        Serial.println("[ERROR_05]");
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_6:
      if(uartString.startsWith("\r\n0000,"))
      {
        Serial.println("[TP 06-1]");
        delay(300);
        initUart();
        bluetoothSerial.print("atz\r");
        currentStep = OPERATION_STEP_7;
        checkNextStep();
      }
      else
      {
        Serial.println("[TP 06-2]");
        delay(300);
        initUart();
        bluetoothSerial.print("at+btkey=0000\r");
        checkNextStep();
      }
    break;
    case OPERATION_STEP_7:
      if(uartString.equals("\r\nOK\r\n"))
      {
        Serial.println("[TP 07]");
        delay(300);
        initUart();
        bluetoothSerial.print("atz\r");
        currentStep = OPERATION_STEP_0;
        checkNextStep();
      }
      else
      {
        Serial.println("[ERROR_07]");
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_8:
      //FB571는 ATZ 명령을 받으면 OK를 출력한다.
      if(uartString.equals("\r\nOK\r\n"))
      {
        Serial.println("[TP 08-1]");
        initUart();
        checkNextStep();
      }
      //FB570은 ATZ 명령을 받으면 OK를 출력하지 않는다.
      else if(uartString.equals("\r\nBTWIN Slave mode start\r\n"))
      {
        Serial.println("[TP 08-2]");
        initUart();
        currentStep = OPERATION_STEP_9;
        checkNextStep();
      }
      else
      {
        Serial.println("[ERROR_08]");
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_9:
      if(uartString.equals("\r\nBTWIN Slave mode start\r\n"))
      {
        Serial.println("[TP 09]");
        initUart();
        checkNextStep();
      }
      else
      {
        Serial.println("[ERROR_09]");
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_10:
      if(uartString.equals("\r\nOK\r\n"))
      {
        Serial.println("[TP 10]");
        delay(300);
        initUart();
        bluetoothSerial.print("at+btscan\r");
        checkNextStep();
      }
      else
      {
        Serial.println("[ERROR_10]");
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;
    case OPERATION_STEP_11:
      if(uartString.equals("\r\nOK\r\n"))
      {
        Serial.println("[TP 11]");
        currentStep++;
      }
      else
      {
        Serial.println("[ERROR_11]");
        initUart();
        currentStep = OPERATION_STEP_ERROR;
      }
    break;

    default:
      if(bluetoothSerial.available())
        Serial.write(bluetoothSerial.read());
      if(Serial.available())
        bluetoothSerial.write(Serial.read());
    break;
  }
}
